@@ -1,13 +1,13 @@
 ;;; marginalia.el --- Enrich existing commands with completion annotations -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2025 Free Software Foundation, Inc.
 
 ;; Author: Omar Antolín Camarena <omar@matem.unam.mx>, Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Omar Antolín Camarena <omar@matem.unam.mx>, Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2020
-;; Version: 1.7
-;; Package-Requires: ((emacs "27.1") (compat "30"))
-;; Homepage: https://github.com/minad/marginalia
+;; Version: 2.0
+;; Package-Requires: ((emacs "28.1") (compat "30"))
+;; URL: https://github.com/minad/marginalia
 ;; Keywords: docs, help, matching, completion
 
 ;; This file is part of GNU Emacs.
@@ -41,7 +41,7 @@
 (defgroup marginalia nil
   "Enrich existing commands with completion annotations."
   :link '(info-link :tag "Info Manual" "(marginalia)")
-  :link '(url-link :tag "Homepage" "https://github.com/minad/marginalia")
+  :link '(url-link :tag "Website" "https://github.com/minad/marginalia")
   :link '(emacs-library-link :tag "Library Source" "marginalia.el")
   :group 'help
   :group 'docs
@@ -136,7 +136,7 @@ determine it."
     ("\\<color\\>" . color)
     ("\\<face\\>" . face)
     ("\\<environment variable\\>" . environment-variable)
-    ("\\<function\\|hook to remove\\>" . function)
+    ("\\<function\\|\\(?:hook\\|advice\\) to remove\\>" . function)
     ("\\<variable\\>" . variable)
     ("\\<input method\\>" . input-method)
     ("\\<charset\\>" . charset)
@@ -304,6 +304,7 @@ The value of `this-command' is used as key for the lookup."
 (declare-function bookmark-prop-get "bookmark")
 
 (declare-function project-current "project")
+(declare-function project-root "project")
 
 (defvar package--builtins)
 (defvar package-archive-contents)
@@ -321,7 +322,10 @@ The value of `this-command' is used as key for the lookup."
 ;;;; Marginalia mode
 
 (defalias 'marginalia--orig-completion-metadata-get
-  (symbol-function (compat-function completion-metadata-get))
+  (symbol-function
+   (if (fboundp 'marginalia--orig-completion-metadata-get)
+       'marginalia--orig-completion-metadata-get
+     (compat-function completion-metadata-get)))
   "Original `completion-metadata-get' function.")
 
 (defvar marginalia--pangram "Cwm fjord bank glyphs vext quiz.")
@@ -400,7 +404,13 @@ FACE is the name of the face, with which the field should be propertized."
   (setq field (if format `(format ,format ,field) `(or ,field "")))
   (when width (setq field `(format ,(format "%%%ds" (- width)) ,field)))
   (when truncate (setq field `(marginalia--truncate ,field ,truncate)))
-  (when face (setq field `(propertize ,field 'face ,face)))
+  (when face
+    (setq field (if (or format width truncate)
+                    (cl-with-gensyms (f)
+                      `(let ((,f ,field))
+                         (put-text-property 0 (length ,f) 'face ,face ,f)
+                         ,f))
+                  `(propertize ,field 'face ,face))))
   field)
 
 (defmacro marginalia--fields (&rest fields)
@@ -502,6 +512,7 @@ l local (L modified compared to default value)
 & alias
 
 Other:
+G custom group
 a face
 t cl-type"
   (let ((class
@@ -535,7 +546,7 @@ t cl-type"
                                  (default-value s))))
                    '("L" . "local, modified from global")
                  '("l" . "local, unmodified")))
-             (if (custom-variable-p s)
+             (if (get s 'standard-value)
                  (if (ignore-errors
                        (not (equal (symbol-value s)
                                    (eval (car (get s 'standard-value))))))
@@ -546,6 +557,7 @@ t cl-type"
                   (cons "&" (format "alias for `%s'" (ignore-errors (indirect-variable s)))))
              (and (get s 'byte-obsolete-variable) '("-" . "obsolete"))))
           (list
+           (and (get s 'group-documentation) '("G" . "custom group"))
            (and (facep s) '("a" . "face"))
            (and (get s 'cl--class) '("t" . "cl-type")))))) ;; cl-find-class, cl--find-class
     (setq class (delq nil class))
@@ -565,7 +577,7 @@ t cl-type"
 ;; Derived from elisp-get-fnsym-args-string
 (defun marginalia--function-args (sym)
   "Return function arguments for SYM."
-  (let ((tmp))
+  (let (tmp)
     (elisp-function-argstring
      (cond
       ((listp (setq tmp (gethash (indirect-function sym)
@@ -574,15 +586,12 @@ t cl-type"
       ((setq tmp (help-split-fundoc
                   (ignore-errors (documentation sym t))
                   sym))
-       (substitute-command-keys (car tmp)))
+       (car tmp))
       ((setq tmp (help-function-arglist sym))
-       (and
-        (if (and (stringp tmp)
-                 (string-search "Arg list not available" tmp))
-            ;; A shorter text fits better into the
-            ;; limited Marginalia space.
-            "[autoload]"
-          tmp)))))))
+       (if (and (stringp tmp) (string-search "not available" tmp))
+           ;; A shorter text fits better into the limited Marginalia space.
+           "[autoload]"
+         tmp))))))
 
 (defun marginalia-annotate-symbol (cand)
   "Annotate symbol CAND with its documentation string."
@@ -676,7 +685,12 @@ keybinding since CAND includes it."
          (format (propertize "#'%s" 'face 'marginalia-function) val))
         ((pred recordp) (format (propertize "#<record %s>" 'face 'marginalia-value) (type-of val)))
         ((pred symbolp) (propertize (symbol-name val) 'face 'marginalia-symbol))
-        ((pred numberp) (propertize (number-to-string val) 'face 'marginalia-number))
+        ((pred numberp)
+         (propertize (number-to-string val)
+                     'face 'marginalia-number
+                     'help-echo (and (integerp val)
+                                     (format "%d, #o%o, #x%x%s" val val val
+                                             (if (characterp val) (format ", ?%c" val) "")))))
         (_ (let ((print-escape-newlines t)
                  (print-escape-control-characters t)
                  ;;(print-escape-multibyte t)
@@ -1048,9 +1062,7 @@ These annotations are skipped for remote paths."
                         prompt)
                        (match-string 1 prompt)))
                 (when-let (proj (project-current))
-                  (cond
-                   ((fboundp 'project-root) (project-root proj))
-                   ((fboundp 'project-roots) (car (project-roots proj))))))))
+                  (project-root proj)))))
     marginalia--project-root))
 
 (defun marginalia-annotate-project-file (cand)
@@ -1171,8 +1183,8 @@ These annotations are skipped for remote paths."
   "Return original category reported by completion metadata."
   ;; Bypass our `marginalia--completion-metadata-get' advice.
   (when-let (cat (marginalia--orig-completion-metadata-get marginalia--metadata 'category))
-    ;; Ignore Emacs 28 symbol-help category in order to ensure that the
-    ;; categories are refined to our categories function and variable.
+    ;; Ignore `symbol-help' category in order to ensure that the categories are
+    ;; refined to our categories function and variable.
     (and (not (eq cat 'symbol-help)) cat)))
 
 (defun marginalia-classify-symbol ()
@@ -1226,8 +1238,7 @@ completion UIs like Vertico or Icomplete."
    (when-let (align (text-property-any 0 (length ann) 'marginalia--align t ann))
      (setq marginalia--cand-width-max
            (max marginalia--cand-width-max
-                (* (ceiling (+ (string-width cand)
-                               (compat-call string-width ann 0 align))
+                (* (ceiling (+ (string-width cand) (string-width ann 0 align))
                             marginalia--cand-width-step)
                    marginalia--cand-width-step)))))
   (cl-loop
@@ -1241,7 +1252,7 @@ completion UIs like Vertico or Icomplete."
                    ('center `(+ center ,marginalia-align-offset))
                    ('left `(+ left ,(+ marginalia-align-offset marginalia--cand-width-max)))
                    ('right `(+ right ,(+ marginalia-align-offset 1
-                                         (- (compat-call string-width ann 0 align)
+                                         (- (string-width ann 0 align)
                                             (string-width ann)))))))
         ann))
      (list cand "" ann))))
@@ -1304,6 +1315,7 @@ Remember `this-command' for `marginalia-classify-by-command-name'."
   ;; As a small optimization we track the base position only for file
   ;; completions, since `marginalia--full-candidate' is currently used only by
   ;; the file annotation function.
+  ;; bug#75910: category instead of `minibuffer-completing-file-name'
   (when minibuffer-completing-file-name
     (let ((base (or (cdr (last completions)) 0)))
       (unless (= marginalia--base-position base)
@@ -1333,6 +1345,8 @@ Remember `this-command' for `marginalia-classify-by-command-name'."
 ;;;###autoload
 (defun marginalia-cycle ()
   "Cycle between annotators in `marginalia-annotator-registry'."
+  ;; Only show `marginalia-cycle' in M-x in recursive minibuffers
+  (declare (completion (lambda (&rest _) (> (minibuffer-depth) 1))))
   (interactive)
   (with-current-buffer (window-buffer
                         (or (active-minibuffer-window)
@@ -1357,9 +1371,5 @@ Remember `this-command' for `marginalia-classify-by-command-name'."
         (setcdr ann (append (cddr ann) (list (cadr ann)))))
       (message "Marginalia: Use annotator `%s' for category `%s'" (cadr ann) (car ann)))))
 
-;; Emacs 28: Only show `marginalia-cycle' in M-x in recursive minibuffers
-(put #'marginalia-cycle 'completion-predicate
-     (lambda (&rest _) (> (minibuffer-depth) 1)))
-
 (provide 'marginalia)
 ;;; marginalia.el ends here
